/***************************************************************************
                          kactiveobjectproxy.cpp  -  description
                             -------------------
    begin                : Fri Nov 22 2002
    copyright            : (C) 2002 by Michael Schuerig
    email                : michael@schuerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kapplication.h>
#include <kdebug.h>
#include <qevent.h>
#include <qstring.h>
#include <qthread.h>

#include "kactiveobjectproxy.h"


KActiveObjectProxy::SlotRequest::SlotRequest() {}
KActiveObjectProxy::SlotRequest::~SlotRequest() {}
KActiveObjectProxy::SignalRequest::SignalRequest()
 : QCustomEvent( QEvent::User + 1001 ) {}
KActiveObjectProxy::SignalRequest::~SignalRequest() {}



KActiveObjectProxy::KActiveObjectProxy( QObject* obj,
                                        QObject* parent,
                                        const char* name )
    : QObject( parent, name ),
      obj_( obj ),
      stopped_( false )
{
    init( true );
}


KActiveObjectProxy::KActiveObjectProxy( QObject* obj,
                                        bool autoStart,
                                        QObject* parent,
                                        const char *name )
    : QObject( parent, name ),
      obj_( obj ),
      stopped_( false )
{
    init( autoStart );
}


void
KActiveObjectProxy::init( bool autoStart )
{
    hasIdleSlot_ = connect( this, SIGNAL(objIdle()),
                            obj_, SLOT(idle()) );

    bool b = connect( kapp, SIGNAL(shutDown()), this, SLOT(stop()) );
    Q_ASSERT( b );

    if ( autoStart )
        start();
}


KActiveObjectProxy::~KActiveObjectProxy()
{
    stop();
    wait();
//###    delete obj_;
}


QObject*
KActiveObjectProxy::object() const
{
    return obj_;
}


void
KActiveObjectProxy::stop()
{
    kdDebug() << "STOP" << endl;
    requestsMutex_.lock();
    bool wasStopped = stopped_;
    stopped_ = true;
    requestsMutex_.unlock();
    if ( !wasStopped )
        hasRequests_.wakeAll();
}


void
KActiveObjectProxy::enqueueSlot( SlotRequest* request )
{
    requestsMutex_.lock();
    kdDebug() << "got lock for appending request" << endl;
    requests_.append( request );
    requestsMutex_.unlock();
    hasRequests_.wakeAll();
}


void
KActiveObjectProxy::enqueueSignal( SignalRequest* request )
{
    postEvent( this, request );
}


void
KActiveObjectProxy::customEvent( QCustomEvent* event )
{
    kdDebug() << "customEvent" << endl;

    SignalRequest* request = dynamic_cast<SignalRequest*>( event );
    if ( request ) {
        request->doEmit( this );
    }
}


void
KActiveObjectProxy::run()
{
    bool gotLock;
    SlotRequest* request;

    while (1) {
        gotLock = false;
        request = 0;

        if ( hasIdleSlot_ ) {
            gotLock = requestsMutex_.tryLock();
        }
        else {
            requestsMutex_.lock();
            gotLock = true;
        }

        if ( gotLock ) {
            kdDebug() << "got lock for de-queueing request" << endl;

            if ( !hasIdleSlot_ ) {
                while ( !stopped_ && requests_.isEmpty() ) {
                    kdDebug() << "waiting for request" << endl;
                    hasRequests_.wait( &requestsMutex_ );
                    kdDebug() << "woke up" << endl;
                }
            }

            if ( !requests_.isEmpty() ) {
                request = requests_.take(0);
            }
            requestsMutex_.unlock();
        }

        if ( stopped_ ) {
            kdDebug() << "was stopped" << endl;
            break;
        }

        if ( request ) {
            kdDebug() << "got request" << endl;
            (*request)();
            delete request;
        }
        else if ( hasIdleSlot_ ) {
            kdDebug() << "idling" << endl;
            emit objIdle();
        }
    }
}

