/***************************************************************************
                          kactiveobjectproxy.h  -  description
                             -------------------
    begin                : Fri Nov 22 2002
    copyright            : (C) 2002 by Michael Schuerig
    email                : michael@schuerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KACTIVEOBJECTPROXY_H
#define KACTIVEOBJECTPROXY_H

#define KACTIVEOBJECTPROXY_H

#include <qmutex.h>
#include <qobject.h>
#include <qptrlist.h>
#include <qthread.h>
#include <qwaitcondition.h>

class QCustomEvent;
class QString;


class KActiveObjectProxy : virtual public QObject, public QThread {
Q_OBJECT
public:
    KActiveObjectProxy( QObject* obj, QObject* parent = 0, const char* name = 0 );
    KActiveObjectProxy( QObject* obj, bool autoStart, QObject* parent = 0, const char* name = 0 );
    virtual ~KActiveObjectProxy() = 0;

signals:
    void objIdle();

public slots:
    void stop();

protected:
    class SlotRequest {
    public:
        SlotRequest();
        virtual ~SlotRequest() = 0;
        virtual void operator()() = 0;
    };

    class SignalRequest : public QCustomEvent {
    public:
        SignalRequest();
        virtual ~SignalRequest() = 0;
        virtual void doEmit( KActiveObjectProxy* obj ) = 0;
    };

    void run();
    void customEvent( QCustomEvent* event );
    void enqueueSlot( SlotRequest* request );
    void enqueueSignal( SignalRequest* request );
    QObject* object() const;

private:
    QPtrList<SlotRequest> requests_;

    friend class SignalRequest;

    void init( bool autoStart );

    QMutex requestsMutex_;
    QWaitCondition hasRequests_;
    QObject* obj_;

    bool stopped_;
    bool hasIdleSlot_;
};

#endif
