/***************************************************************************
                          ksmsapp.cpp  -  description
                             -------------------
    begin                : Sat Oct 26 2002
    copyright            : (C) 2002 by Michael Schuerig
    email                : michael@schuerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <dcopclient.h>
#include <kdebug.h>
#include <kglobal.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <krun.h>
#include <kstandarddirs.h>
#include <qglobal.h>

#include "ksmsarchive.h"       
#include "ksmsiface.h"
#include "ksmsmsg.h"
#include "ksmsprefs.h"
#include "ksmsprefsdialog.h"
#include "messagestore.h"
#include "mobilephonebook.h"
#include "phoneaccess.h"
#include "phoneutils.h"
#include "senderfactory.h"
#include "ksmsapp.h"


namespace {

const QString ADDRESSBOOK_APP( "kaddressbook" );

bool
checkForAddressBook()
{
    if ( KStandardDirs::findExe( ADDRESSBOOK_APP ).isEmpty() ) {
        KMessageBox::information( 0,
            i18n("No external address book application found. You might want to "
                 "install KAddressBook from the kdepim module.") );
        return false;
    }
    else {
        return true;
    }
}


PhoneAccess::Config
phoneAccessConfigFromPrefs()
{
    KSmsPrefs* prefs = KSmsPrefs::instance();
    
    return PhoneAccess::Config( prefs->phone_device,
                                prefs->phone_baudrate,
                                prefs->phone_timeout,
                                prefs->phone_initString,
                                prefs->phone_smsStore );
}

}

class KSmsApp::DCOPDelegate : public KSmsIface {
public:
    DCOPDelegate( KSmsApp* app ) : app_(app) {}
    virtual void newMessage() const
        { app_->newMessage(); }
private:
    KSmsApp* app_;
};



KSmsApp::KSmsApp()
    : prefsDialog_(0),
      phoneBook_( new MobilePhoneBook(KGlobal::config()) ),
      messagesFile_( locateLocal("appdata", "messages") )
{
    Q_ASSERT( phoneBook_ != 0 );
    Q_ASSERT( !messagesFile_.isEmpty() );

    PhoneUtils::setLocalCountryPrefix( KSmsPrefs::instance()->phone_countryPrefix );

    // SenderFactory needs country prefix to be set
    messageStore_ = new MessageStore( new SenderFactory(),
                                      messagesFile_ );
    Q_ASSERT( messageStore_ != 0 );

    PhoneAccess::instance()->setConfig( phoneAccessConfigFromPrefs() );

    
    // If app is a KUniqueApplication, it's already registered.
    if ( !dcopClient()->isRegistered() ) {
        dcopClient()->registerAs( name() );
    }
    dcopDelegate_ = new DCOPDelegate( this );
    dcopClient()->setDefaultObject( dcopDelegate_->objId() );

}


KSmsApp::~KSmsApp()
{
    phoneBook_->saveConfig( KGlobal::config() );
    messageStore_->saveMessages();
    KSmsPrefs::instance()->writeConfig();

    if ( prefsDialog_ )
        delete prefsDialog_;

    delete dcopDelegate_;
}


void
KSmsApp::showMainWindow()
{
    KSmsArchive* mainWin = new KSmsArchive();
    mainWin->show();
}


void
KSmsApp::newMessage() const
{
    newMessage( QString::null );
}


void
KSmsApp::newMessage( const QString& addressee ) const
{
    KSmsMsg* msgWin = new KSmsMsg();
    msgWin->setAddressee( addressee );
    msgWin->show();
}


void
KSmsApp::openAddressBook( const QString& uid ) const
{
    if ( checkForAddressBook() ) {
        if ( uid.isEmpty() ) {
            KRun::runCommand( ADDRESSBOOK_APP );
        }
        else {
            KRun::runCommand( ADDRESSBOOK_APP + " --editor-only --uid " + uid );
        }
    }
}


void
KSmsApp::configurePreferences()
{
    kdDebug() << "KSmsApp::configurePreferences" << endl;

    if ( !prefsDialog_ ) {
        prefsDialog_ = new KSmsPrefsDialog();
        prefsDialog_->readConfig();
        connect( prefsDialog_, SIGNAL(configChanged()),
                 this, SLOT(slotConfigChanged()) );
    }

    prefsDialog_->show();
    prefsDialog_->raise();
}


void
KSmsApp::slotConfigChanged()
{
    PhoneAccess::instance()->setConfig( phoneAccessConfigFromPrefs() );
    
    emit configChanged();
}


KSharedPtr<MobilePhoneBook>
KSmsApp::phoneBook()
{
    return phoneBook_;
}


KSharedPtr<MessageStore>
KSmsApp::messageStore()
{
    return messageStore_;
}

