/***************************************************************************
                          ksmsarchivedoc.cpp  -  description
                             -------------------
    begin                : Mon Oct 28 2002
    copyright            : (C) 2002 by Michael Schuerig
    email                : michael@schuerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kdebug.h>
#include <kglobal.h>
#include <qglobal.h>
#include <qstring.h>
 
#include "messagestore.h"
#include "mobilephonebook.h"


#include "ksmsarchivedoc.h"



KSmsArchiveDoc::KSmsArchiveDoc( KSharedPtr<MobilePhoneBook> phoneBook,
                                KSharedPtr<MessageStore> messageStore,
                                QObject* parent, const char* name )
    : QObject( parent, name ),
      phoneBook_( phoneBook ),
      messageStore_( messageStore )
{
    Q_ASSERT( phoneBook_ != 0 );
    Q_ASSERT( messageStore_ != 0 );

    connect( messageStore_, SIGNAL(messagesChangedForSender(const MsgSender*)),
             this, SLOT(emitMessagesChangedForSender(const MsgSender*)) );
    connect( messageStore_, SIGNAL(senderWasAdded(const MsgSender*)),
             this, SLOT(emitSendersChanged()) );
    connect( messageStore_, SIGNAL(storeChanged()),
             this, SLOT(emitSendersChanged()) );
}


KSmsArchiveDoc::~KSmsArchiveDoc()
{
}


const MsgSenderList
KSmsArchiveDoc::senders() const
{
    return messageStore_->senders();
}


uint
KSmsArchiveDoc::messageCount() const
{
    return messageStore_->messageCount();
}


void
KSmsArchiveDoc::importMessagesFromPhone()
{
    connect( messageStore_, SIGNAL(progress(unsigned long, unsigned long)),
             this, SLOT(emitProgress(unsigned long, unsigned long)) );

    messageStore_->importMessagesFromPhone();
    
    disconnect( messageStore_, SIGNAL(progress(unsigned long, unsigned long)),
                this, SLOT(emitProgress(unsigned long, unsigned long)) );
}


void
KSmsArchiveDoc::readMessages()
{
    connect( messageStore_, SIGNAL(progress(unsigned long, unsigned long)),
             this, SLOT(emitProgress(unsigned long, unsigned long)) );

    messageStore_->readMessages();

    disconnect( messageStore_, SIGNAL(progress(unsigned long, unsigned long)),
                this, SLOT(emitProgress(unsigned long, unsigned long)) );
}


void
KSmsArchiveDoc::saveMessages() const
{
    messageStore_->saveMessages();
}


void
KSmsArchiveDoc::emitMessagesChangedForSender( const MsgSender* sender )
{
    emit messagesChangedForSender( sender );
}


void
KSmsArchiveDoc::deleteMessage( const Message* msg )
{
    kdDebug() << "KSmsArchiveDoc::deleteMessage: " << msg->toString() << endl;
    messageStore_->deleteMessage( const_cast<Message*>(msg) );
}


void
KSmsArchiveDoc::emitSendersChanged()
{
    emit sendersChanged();
}


void
KSmsArchiveDoc::emitProgress( unsigned long current,
                              unsigned long max )
{
    emit progress( current, max );
}

