/***************************************************************************
                          ksmsmsg.cpp  -  description
                             -------------------
    begin                : Sat Oct 26 2002
    copyright            : (C) 2002 by Michael Schuerig
    email                : michael@schuerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <klocale.h>
#include <kaction.h>
#include <kapplication.h>
#include <kdebug.h>
#include <kedittoolbar.h>
#include <kkeydialog.h>
//#include <kmessagebox.h>
#include <kstdaction.h>
#include <kstatusbar.h>
#include <qglobal.h>

#include "ksmsapp.h"
#include "ksmsmsgdoc.h"
#include "ksmsmsgview.h"
#include "messagestore.h"
#include "mobilephonebook.h"
#include "ksmsmsg.h"


const QString KSmsMsg::UI_RESOURCE_FILE( "ksmsmsgui.rc" );
const QString KSmsMsg::WINDOW_NAME( "MessageWindow" );



KSmsMsg::KSmsMsg()
    : KMainWindow(0),
      document_(0), view_(0), newMsgAction_(0), sendMsgAction_(0),
      toolbarAction_(0), statusbarAction_(0)
{
    setWFlags( Qt::WDestructiveClose );
    resize( 280, 280 );

    initDocument();
    initView();
    initStatusBar();
    initActions();
    setAutoSaveSettings( WINDOW_NAME );

    toolbarAction_->setChecked( !toolBar()->isHidden() );
    statusbarAction_->setChecked( !statusBar()->isHidden() );
}


KSmsMsg::~KSmsMsg()
{
}


void
KSmsMsg::initDocument()
{
    document_ = new KSmsMsgDoc( this, "Message Doc" );
    connect( document_, SIGNAL(messageWasSent(const QString&, const QString&)),
             application()->phoneBook(), SLOT(rememberAsRecentNumber(const QString&)) );
    connect( document_, SIGNAL(messageWasSent(const QString&, const QString&)),
             application()->messageStore(),
             SLOT(addSentMessage(const QString&, const QString&)) );
    connect( document_, SIGNAL(messageWasSent(const QString&, const QString&)),
             this, SLOT(close()) );
}


void
KSmsMsg::initView()
{
    Q_ASSERT( document_ != 0 );
    view_ = new KSmsMsgView( document_,
                             application()->phoneBook(),
                             this, "Message View" );
    setCentralWidget(view_);
}


void
KSmsMsg::initStatusBar()
{
    Q_ASSERT( document_ != 0 );
    statusBar()->insertItem( i18n("Ready."), ID_STATUS_GENERAL, 1 );
    statusBar()->setItemAlignment( ID_STATUS_GENERAL, AlignLeft | AlignVCenter );
    statusBar()->insertItem( i18n(" Length: %1 ").arg(0), ID_STATUS_MSG_LENGTH, 0, true );
    connect( document_, SIGNAL(textChanged(const QString&)),
             this, SLOT(updateStatus(const QString&)) );


    actionCollection()->setHighlightingEnabled( true );
    connect( actionCollection(), SIGNAL(actionStatusText(const QString&)),
             this, SLOT(statusMessage(const QString&)) );
    connect( actionCollection(), SIGNAL(clearStatusText()),
             this, SLOT(statusMessage()) );
}


void
KSmsMsg::initActions()
{
    Q_ASSERT( document_ != 0 );
    Q_ASSERT( view_ != 0 );

    KAction* action;

    newMsgAction_ = new KAction( i18n("&New Message"), "filenew", CTRL + Key_N,
                                 application(), SLOT(newMessage()), actionCollection(), "new_message" );
    Q_CHECK_PTR( newMsgAction_ );
    newMsgAction_->setToolTip( i18n("Creates a new message.") );

    KStdAction::close( this, SLOT(close()), actionCollection() );

    sendMsgAction_ = new KAction( i18n("&Send Message"), "mail_send", CTRL + Key_Return,
                                  document_, SLOT(sendMessage()), actionCollection(), "send_message" );
    Q_CHECK_PTR( sendMsgAction_ );
    sendMsgAction_->setToolTip( i18n("Sends this message.") );
    connect( document_, SIGNAL(canBeSent(bool)), sendMsgAction_, SLOT(setEnabled(bool)) );

    action = new KAction( i18n("Address Book..."), "contents", 0,
                          application(), SLOT(openAddressBook()), actionCollection(),
                          "open_addressbook" );
    Q_CHECK_PTR( action );
    action->setToolTip( i18n("Open the address book") );

    action = KStdAction::undo( view_, SLOT(undo()), actionCollection() );
    connect( view_, SIGNAL(undoAvailable(bool)), action, SLOT(setEnabled(bool)) );
    action = KStdAction::redo( view_, SLOT(redo()), actionCollection() );
    connect( view_, SIGNAL(redoAvailable(bool)), action, SLOT(setEnabled(bool)) );
    action = KStdAction::cut( view_, SLOT(cut()), actionCollection() );
    connect( view_, SIGNAL(copyAvailable(bool)), action, SLOT(setEnabled(bool)) );
    action = KStdAction::copy( view_, SLOT(copy()), actionCollection() );
    connect( view_, SIGNAL(copyAvailable(bool)), action, SLOT(setEnabled(bool)) );
    action = KStdAction::paste( view_, SLOT(paste()), actionCollection() );
    action = KStdAction::selectAll( view_, SLOT(selectAll()), actionCollection() );
    action = KStdAction::deselect( view_, SLOT(deselect()), actionCollection() );
    connect( view_, SIGNAL(copyAvailable(bool)), action, SLOT(setEnabled(bool)) );
      
    KStdAction::spelling( document_, SLOT(checkSpelling()), actionCollection() );

    toolbarAction_ = KStdAction::showToolbar( this, SLOT(updateToolbar()), actionCollection() );
    Q_CHECK_PTR( toolbarAction_ );
    
    statusbarAction_ = KStdAction::showStatusbar( this, SLOT(updateStatusBar()), actionCollection() );
    Q_CHECK_PTR( statusbarAction_ );

    KStdAction::keyBindings( this, SLOT(configureKeyBindings()), actionCollection() );
    KStdAction::configureToolbars( this, SLOT(configureToolbars()), actionCollection() );

    createGUI( UI_RESOURCE_FILE );
}


void
KSmsMsg::setAddressee( const QString& addressee )
{
    kdDebug() << "KSmsMsg::setAddressee: " << addressee << endl;
    
//#### set addressee in doc and have doc update view
}


void
KSmsMsg::setText( const QString& text )
{
    kdDebug() << "KSmsMsg::setText: " << text << endl;

//#### set addressee in doc and have doc update view
}


KSmsApp*
KSmsMsg::application() const
{
    return dynamic_cast<KSmsApp*>(KApplication::kApplication());
}


void
KSmsMsg::updateStatus( const QString& messageText )
{
    statusBar()->changeItem( i18n(" Length: %1 ").arg(messageText.length()),
                             ID_STATUS_MSG_LENGTH );
}


void
KSmsMsg::statusMessage( const QString& msg )
{
    statusBar()->changeItem( msg, ID_STATUS_GENERAL );
}


void
KSmsMsg::close()
{
    KMainWindow::close(true);
}


bool
KSmsMsg::queryClose()
{
    kdDebug() << "KSmsMsg::queryClose" << endl;

    saveMainWindowSettings( KGlobal::config(), WINDOW_NAME );
    return true;
}


bool
KSmsMsg::queryExit()
{
    kdDebug() << "KSmsMsg::queryExit" << endl;

    saveMainWindowSettings( KGlobal::config(), WINDOW_NAME );
    return true;
}


void
KSmsMsg::updateToolbar()
{
    if ( toolbarAction_->isChecked() ) {
        toolBar()->show();
    }
    else {
        toolBar()->hide();
    }
}


void
KSmsMsg::updateStatusBar()
{
    if ( statusbarAction_->isChecked() ) {
        statusBar()->show();
    }
    else {
        statusBar()->hide();
    }
}


void
KSmsMsg::configureToolbars()
{
    kdDebug() << "KSmsMsg::configureToolbars" << endl;

    saveMainWindowSettings( KGlobal::config(), WINDOW_NAME );
    KEditToolbar dlg( actionCollection(), UI_RESOURCE_FILE );
    connect( &dlg, SIGNAL(newToolbarConfig()), this, SLOT(newToolbarConfig()) );
    if ( dlg.exec() ) {
        createGUI( UI_RESOURCE_FILE );
    }
}


void
KSmsMsg::newToolbarConfig()
{
    applyMainWindowSettings( KGlobal::config(), WINDOW_NAME );
}



void
KSmsMsg::configureKeyBindings()
{
    kdDebug() << "configureKeyBindings" << endl;

    KKeyDialog::configure(actionCollection(), xmlFile());
}

