/***************************************************************************
                          ksmsmsgdoc.cpp  -  description
                             -------------------
    begin                : Sat Oct 26 2002
    copyright            : (C) 2002 by Michael Schuerig
    email                : michael@schuerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kdebug.h>
#include <kspell.h>
#include <qglobal.h>
 
#include "phoneutils.h"
#include "threadedphoneaccess.h"
#include "ksmsmsgdoc.h"


enum { MAX_TEXT_LENGTH = 160 };


KSmsMsgDoc::KSmsMsgDoc( bool allowEmptyMessage,
                        QObject* parent,
                        const char* name )
    : QObject( parent, name ),
      allowEmptyMessage_( allowEmptyMessage )
{
}


KSmsMsgDoc::KSmsMsgDoc( QObject* parent,
                        const char* name )
    : QObject( parent, name ),
      allowEmptyMessage_( false )
{
}


KSmsMsgDoc::~KSmsMsgDoc()
{
}


bool
KSmsMsgDoc::allowEmptyMessage()
{
    return allowEmptyMessage_;
}


void
KSmsMsgDoc::setAllowEmptyMessage( bool allow)
{
    allowEmptyMessage_ = allow;
}


void
KSmsMsgDoc::checkSpelling()
{
    kdDebug() << "KSmsMsgDoc::checkSpelling()" << endl;

    if ( KSpell::modalCheck(text_) ) {
        emit textChanged( text_ );
    }
}


void
KSmsMsgDoc::sendMessage()
{
    kdDebug() << "KSmsMsgDoc::sendMessage()" << endl;
    Q_ASSERT( isMessageValid() );

    const QString normNumber( PhoneUtils::normalizePhoneNumber(phoneNumber_) );
    
    kdDebug() << "    number: " << normNumber << endl;
    kdDebug() << "    text: " << text_ << endl;

    ThreadedPhoneAccess::instance()->sendMessage( normNumber, text_ );

    emit messageWasSent( normNumber, text_ );
}


void
KSmsMsgDoc::setText( const QString& text )
{
//    kdDebug() << "KSmsMsgDoc::setText: " << text << endl;

    text_ = text;
    emit canBeSent( isMessageValid() );
    emit textChanged( text_ );
}


void
KSmsMsgDoc::setPhoneNumber( const QString& phoneNumber )
{
//    kdDebug() << "KSmsMsgDoc::setPhoneNumber: " << phoneNumber << endl;
    
    phoneNumber_ = phoneNumber;
    emit canBeSent( isMessageValid() );
}


bool
KSmsMsgDoc::isMessageValid()
{
    return isTextValid() && isPhoneNumberValid();
}


bool
KSmsMsgDoc::isTextValid()
{
    return (allowEmptyMessage_ || !text_.isEmpty())
           && text_.length() <= MAX_TEXT_LENGTH;
}


bool
KSmsMsgDoc::isPhoneNumberValid()
{
    return ! PhoneUtils::normalizePhoneNumber(phoneNumber_).isEmpty();
}
