/***************************************************************************
                          ksmsmsgview.cpp  -  description
                             -------------------
    begin                : Sat Oct 26 2002
    copyright            : (C) 2002 by Michael Schuerig
    email                : michael@schuerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kcombobox.h>
#include <kdebug.h>
#include <kdialog.h>
#include <klocale.h>
#include <qglobal.h>
#include <qpushbutton.h>
#include <ktextedit.h>
#include <qlayout.h>

#include "mobilephonebook.h"
#include "ksmsmsgdoc.h"
#include "ksmsmsgview.h"


namespace {
const QString NAME_NUMBER_TEMPLATE( "%1: %2" );
const QString NUMBER_TEMPLATE( "%1" );
void insertFormatted( KComboBox* box, const MobilePhoneBook::EntryList& entries );
const QString formatNameNumber( const MobilePhoneBook::Entry& entry );
}


KSmsMsgView::KSmsMsgView( KSmsMsgDoc* document,
                          KSharedPtr<MobilePhoneBook> phoneBook,
                          QWidget* parent,
                          const char* name)
    : QWidget(parent, name),
      document_(document), phoneBook_(phoneBook)
{
    Q_ASSERT( document_ != 0 );
    Q_ASSERT( phoneBook_ != 0 );

    initWidgets();
}


KSmsMsgView::~KSmsMsgView()
{
}


void
KSmsMsgView::initWidgets()
{
    QVBoxLayout* layout = new QVBoxLayout( this, KDialog::marginHint(), KDialog::spacingHint() );
    layout->setAutoAdd(true);

    phoneNumberBox_ = new KComboBox( true, this );
    Q_CHECK_PTR( phoneNumberBox_ );
    //### set validator?
    phoneNumberBox_->setInsertionPolicy( KComboBox::NoInsertion );
    updatePhoneNumbers();
    document_->setPhoneNumber( phoneNumberBox_->currentText() );
    connect( phoneNumberBox_, SIGNAL(textChanged(const QString&)),
             document_, SLOT(setPhoneNumber(const QString&)) );
    connect( phoneBook_, SIGNAL(phoneNumbersChanged()),
             this, SLOT(updatePhoneNumbers()) );

    editor_ = new KTextEdit(this);
    editor_->setCheckSpellingEnabled(true);
    Q_CHECK_PTR( editor_ );
    editor_->setTextFormat( PlainText );
    connect( editor_, SIGNAL(textChanged()),
             this, SLOT(viewTextChanged()) );
    connect( editor_, SIGNAL(undoAvailable(bool)),
             this, SLOT(emitUndoAvailable(bool)) );
    connect( editor_, SIGNAL(redoAvailable(bool)),
             this, SLOT(emitRedoAvailable(bool)) );
    connect( editor_, SIGNAL(copyAvailable(bool)),
             this, SLOT(emitCopyAvailable(bool)) );

    sendButton_ = new QPushButton( i18n("Send Message"), this );
    Q_CHECK_PTR( sendButton_ );
    sendButton_->setEnabled( document_->isMessageValid() );
    connect( sendButton_, SIGNAL(clicked()), document_, SLOT(sendMessage()) );
    connect( document_, SIGNAL(canBeSent(bool)), sendButton_, SLOT(setEnabled(bool)) );

    setMinimumSize( sizeHint() );
}


void
KSmsMsgView::updatePhoneNumbers()
{
    kdDebug() << "KSmsMsgView::updatePhoneNumbers" << endl;

    phoneNumberBox_->clear();
    if ( phoneBook_->recentNumbers().isEmpty() ) {
        phoneNumberBox_->insertItem("");
    }
    else {
        phoneNumberBox_->insertStringList( phoneBook_->recentNumbers() );
        phoneNumberBox_->insertItem("------------------------------"); //### real separator
    }
    insertFormatted( phoneNumberBox_, phoneBook_->entries() );
}


void
KSmsMsgView::viewTextChanged()
{
    disconnect( document_, SIGNAL(textChanged(const QString&)),
                this, SLOT(docTextChanged(const QString&)) );

    document_->setText( editor_->text() );

    connect( document_, SIGNAL(textChanged(const QString&)),
             this, SLOT(docTextChanged(const QString&)) );
}


void
KSmsMsgView::docTextChanged( const QString& text )
{
    blockSignals( true );
    editor_->setText( text );
    blockSignals( false );
}


void
KSmsMsgView::undo()
{
    editor_->undo();
}

void
KSmsMsgView::redo()
{
    editor_->redo();
}

void
KSmsMsgView::copy()
{
    editor_->copy();
}

void
KSmsMsgView::cut()
{
    editor_->cut();
}

void
KSmsMsgView::paste()
{
    editor_->paste();
}

void
KSmsMsgView::selectAll()
{
    editor_->selectAll( true );
}

void
KSmsMsgView::deselect()
{
    editor_->selectAll( false );
}


void
KSmsMsgView::emitUndoAvailable( bool yes )
{
    emit undoAvailable( yes );
}

void
KSmsMsgView::emitRedoAvailable( bool yes )
{
    emit redoAvailable( yes );
}

void
KSmsMsgView::emitCopyAvailable( bool yes )
{
    emit copyAvailable( yes );
}


namespace {

const QString
formatNameNumber( const MobilePhoneBook::Entry& entry )
{
    if ( !entry.name.isEmpty() ) {
        return NAME_NUMBER_TEMPLATE.arg(entry.name).arg(entry.phoneNumber);
    }
    else {
        return NUMBER_TEMPLATE.arg(entry.phoneNumber);
    }
}


void
insertFormatted( KComboBox* box, const MobilePhoneBook::EntryList& entries )
{
    for ( MobilePhoneBook::EntryList::const_iterator it = entries.begin();
          it != entries.end(); ++it ) {
        box->insertItem( formatNameNumber(*it) );
    }
}

}
