/***************************************************************************
                          ksmsprefsdialog.cpp  -  description
                             -------------------
    begin                : Fri Nov 15 2002
    copyright            : (C) 2002 by Michael Schuerig
    email                : michael@schuerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kcombobox.h>
#include <kiconloader.h>
#include <klocale.h>
#include <qcheckbox.h>
#include <qframe.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qspinbox.h>
 
#include "ksmsprefs.h"
 
#include "ksmsprefsdialog.h"

namespace {

const QStringList BAUDRATES( QStringList::split(',',
    "300,600,1200,2400,4800,9600,19200,38400,57600,115200,230400,460800") );

}


KSmsPrefsDialog::KSmsPrefsDialog( QWidget* parent,
                                  const char* name,
                                  bool modal )
    : KPrefsDialog( KSmsPrefs::instance(), parent, name, modal )
{
    setupPhoneTab();
    setupGuiTab();
}


KSmsPrefsDialog::~KSmsPrefsDialog()
{
    delete phone_device_;
    delete phone_baudrate_;
    delete phone_timeout_;
    delete phone_smsStore_;
    delete phone_initString_;
    delete phone_countryPrefix_;
    delete ui_dockable_;
    delete ui_docked_;
    delete ui_showPhoneNumber_;
}


void
KSmsPrefsDialog::setupPhoneTab()
{
    QFrame* topFrame = addPage( i18n("Phone"), 0,
                                DesktopIcon("connect_no", KIcon::SizeMedium) );
    QGridLayout* topLayout = new QGridLayout( topFrame, 6, 2 );
    topLayout->setSpacing( spacingHint() );
    topLayout->setMargin( marginHint() );

    phone_device_ = addWidString( i18n("Phone device:"),
                                  &(KSmsPrefs::instance()->phone_device),
                                  topFrame );
    topLayout->addWidget( phone_device_->label(), 0, 0 );
    topLayout->addWidget( phone_device_->lineEdit(), 0, 1 );

    phone_baudrate_ = addWidStringChoice( i18n("Baudrate:"),
                                          &(KSmsPrefs::instance()->phone_baudrate),
                                          BAUDRATES,
                                          topFrame );
    topLayout->addWidget( phone_baudrate_->label(), 1, 0 );
    topLayout->addWidget( phone_baudrate_->comboBox(), 1, 1 );

    phone_timeout_ = addWidInt( i18n("Timeout:"),
                                &(KSmsPrefs::instance()->phone_timeout),
                                topFrame,
                                0, 300 );
    topLayout->addWidget( phone_timeout_->label(), 2, 0 );
    topLayout->addWidget( phone_timeout_->spinBox(), 2, 1 );

    phone_smsStore_ = addWidString( i18n("SMS store:"),
                                    &(KSmsPrefs::instance()->phone_smsStore),
                                    topFrame );
    phone_smsStore_->lineEdit()->setMaxLength( 2 );
    topLayout->addWidget( phone_smsStore_->label(), 3, 0 );
    topLayout->addWidget( phone_smsStore_->lineEdit(), 3, 1 );


    phone_initString_ = addWidString( i18n("Init string:"),
                                      &(KSmsPrefs::instance()->phone_initString),
                                      topFrame );
    topLayout->addWidget( phone_initString_->label(), 4, 0 );
    topLayout->addWidget( phone_initString_->lineEdit(), 4, 1 );

    phone_countryPrefix_ = addWidString( i18n("Country prefix:"),
                                         &(KSmsPrefs::instance()->phone_countryPrefix),
                                         topFrame );
    topLayout->addWidget( phone_countryPrefix_->label(), 5, 0 );
    topLayout->addWidget( phone_countryPrefix_->lineEdit(), 5, 1 );

    topLayout->setRowStretch( 2, 1 );
}


void
KSmsPrefsDialog::setupGuiTab()
{
    QFrame* topFrame = addPage( i18n("User Interface"), 0,
                                DesktopIcon("window_list", KIcon::SizeMedium) );
    QGridLayout* topLayout = new QGridLayout( topFrame, 6, 2 );
    topLayout->setSpacing( spacingHint() );
    topLayout->setMargin( marginHint() );

    ui_dockable_ = addWidBool( i18n("Run in system tray"),
                               &(KSmsPrefs::instance()->ui_dockable),
                               topFrame );
    topLayout->addWidget( ui_dockable_->checkBox(), 0, 0 );

    ui_showPhoneNumber_ = addWidBool( i18n("Show senders' phone number"),
                                      &(KSmsPrefs::instance()->ui_showPhoneNumber),
                                      topFrame );
    topLayout->addWidget( ui_showPhoneNumber_->checkBox(), 1, 0 );

    topLayout->setRowStretch( 2, 1 );
}
