/***************************************************************************
                          message.cpp  -  description
                             -------------------
    begin                : Mon Oct 28 2002
    copyright            : (C) 2002 by Michael Schuerig
    email                : michael@schuerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kdebug.h>
#include <kmdcodec.h>
 
#include "msgsender.h"
#include "message.h"


using namespace MessageConstants;

namespace {

const QString DELIVER_STRING( "deliver" );
const QString SUBMIT_STRING(  "submit" );
const QString UNKNOWN_STRING( "unknown" );

enum { MESSAGE_HASH_SIZE = 9973 };
KMD5 md5;
QCString hashKey( const Message* msg );
QCString hashKey( const QString& phoneNumber, const QDateTime& timestamp,
                  const QString& text );

}


Message::Message( MessageConstants::Type type,
                  const QString& phoneNumber,
                  const MsgSender* sender,
                  const MsgSender* receiver,
                  const QDateTime& timestamp,
                  const QString& text )
    : type_(type), phoneNumber_(phoneNumber),
      sender_(sender), receiver_(receiver),
      timestamp_(timestamp), text_(text)
{
//    kdDebug() << "New Message" << toString();
}


Message::~Message()
{
//#ifndef  NDEBUG
//    kdDebug() << "Message::~Message: " << endl
//              << toString() << endl;
//#endif
}


const QString
Message::typeString() const
{
    switch ( type_ ) {
    case SMS_DELIVER:
        return DELIVER_STRING;
    case SMS_SUBMIT:
        return SUBMIT_STRING;
    default:
        return UNKNOWN_STRING;
    }
}


MessageConstants::Type
Message::messageTypeFromString( const QString& type )
{
    const QString lowerType( type.lower() );
    if (      lowerType == DELIVER_STRING )
        return SMS_DELIVER;
    else if ( lowerType == SUBMIT_STRING )
        return SMS_SUBMIT;
    else
        return SMS_UNKNOWN;
}


const QString
Message::toString() const
{
    return "Sender:    " + sender()->toString() + "\n"
         + "Receiver:  " + receiver()->toString() + "\n"
         + "Timestamp: " + timestamp().toString(Qt::ISODate) + "\n"
         + text() + "\n";
}


MessageContainer::MessageContainer()
    : hashToMessage_( MESSAGE_HASH_SIZE )
{
    setAutoDelete( true );
}


MessageContainer::~MessageContainer()
{
}


Message*
MessageContainer::newMessage( MessageConstants::Type type,
                              const QString& phoneNumber,
                              const MsgSender* sender,
                              const MsgSender* receiver,
                              const QDateTime& timestamp,
                              const QString& text )
{
    Message* msg = new Message( type, phoneNumber,
                                sender, receiver,
                                timestamp, text );

    append( msg );
    hashToMessage_.insert( hashKey(msg), msg );
    
    return msg;
}


void
MessageContainer::removeMessage( Message* msg )
{
    Q_ASSERT( msg != 0 );
    Message* msg2 = hashToMessage_.take( hashKey(msg) );
    Q_ASSERT( msg == msg2 );

    bool removed = removeRef( msg );
    Q_ASSERT( removed );
}


Message*
MessageContainer::find( const QString& phoneNumber,
                        const QDateTime& timestamp,
                        const QString& text )
{
    Message* msg = hashToMessage_.find( hashKey(phoneNumber, timestamp, text) );

    if ( msg == 0 )
        return 0;
    
    if (    timestamp   == msg->timestamp()
         && text        == msg->text()
         && phoneNumber == msg->phoneNumber() ) {
        return msg;
    }

    return 0;
}


namespace {

QCString
hashKey( const Message* msg )
{
    return hashKey( msg->phoneNumber(), msg->timestamp(), msg->text() );
}


QCString
hashKey( const QString& phoneNumber,
         const QDateTime& timestamp,
         const QString& text )
{
    md5.reset();
    md5.update( phoneNumber );
    md5.update( timestamp.toString(Qt::ISODate) );
    md5.update( text );
    return md5.hexDigest();
}

}
