/***************************************************************************
                          message.h  -  description
                             -------------------
    begin                : Mon Oct 28 2002
    copyright            : (C) 2002 by Michael Schuerig
    email                : michael@schuerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MESSAGE_H
#define MESSAGE_H

#include <qasciidict.h>
#include <qdatetime.h>
#include <qstring.h>
#include <qptrlist.h>

#include "messageconstants.h"

class MsgSender;


/**
 * @author Michael Schuerig
 */
class Message {

    friend class MessageList;
    friend class QPtrList<Message>;
    friend class MessageContainer;
    friend class QAsciiDict<Message>;

public:

    static MessageConstants::Type messageTypeFromString( const QString& type );

    MessageConstants::Type type() const { return type_; }
    const MsgSender* sender() const { return sender_; }
    const MsgSender* receiver() const { return receiver_; }
    const QString typeString() const;
    const QString phoneNumber() const { return phoneNumber_; }
    const QDateTime timestamp() const { return timestamp_; }
    const QString text() const { return text_; }

    const QString toString() const;
            
private:
    Message( MessageConstants::Type type,
             const QString& phoneNumber,
             const MsgSender* sender,
             const MsgSender* receiver,
             const QDateTime& timestamp,
             const QString& text );
    ~Message();

    Message(); // not implemented
    Message( const Message& ); // not implemented
    Message& operator=( const Message& ); // not implemented

    const MessageConstants::Type type_;
    const QString phoneNumber_;
    const MsgSender* sender_;
    const MsgSender* receiver_;
    const QDateTime timestamp_;
    const QString text_;
};


struct MessageList : public QPtrList<Message> {
};


class MessageContainer : public MessageList {
public:
    MessageContainer();
    ~MessageContainer();
    
    Message* newMessage( MessageConstants::Type type,
                         const QString& phoneNumber,
                         const MsgSender* sender,
                         const MsgSender* receiver,
                         const QDateTime& timestamp,
                         const QString& text );
    /**
     * @param phoneNumber Normalized phone number.
     */
    Message* find( const QString& phoneNumber, const QDateTime& timestamp,
                   const QString& text );

    void removeMessage( Message* msg );
    
private:
    QAsciiDict<Message> hashToMessage_;
};


#endif
