/***************************************************************************
                          messageformatter.cpp  -  description
                             -------------------
    begin                : Mon Nov 18 2002
    copyright            : (C) 2002 by Michael Schuerig
    email                : michael@schuerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kdebug.h>
#include <kglobal.h>
#include <klocale.h>
#include <qregexp.h>
#include <qstring.h>

#include "message.h"
#include "msgsender.h"
#include "messageformatter.h"

namespace {

const QRegExp SENDER_TAG = i18n( "%sender" );
const QRegExp RECEIVER_TAG = i18n( "%receiver" );
const QRegExp DATE_TAG = i18n( "%date" );
const QRegExp TEXT_TAG = i18n( "%text" );

const QString formatAddress( const MsgSender* address );
const QString formatTimestamp( const QDateTime ts );
const QString formatText( const QString& text, const QString& lineSep );
const QString substituteTags( const Message* msg, const QString& templ, const QString& lineSep );

}


MessageFormatter::MessageFormatter( const QString& msgTemplate,
                                    const QString& lineSep )
    : msgTemplate_( msgTemplate ),
      lineSep_( lineSep )
{
}


MessageFormatter::~MessageFormatter()
{
}


const QString
MessageFormatter::format( const Message* msg ) const
{
    return substituteTags( msg, msgTemplate_, lineSep_ );
}


namespace {

const QString
formatAddress( const MsgSender* address )
{
    if ( !address->name().isEmpty() )
        return address->name();
    else
        return address->phoneNumber();
}


const QString
formatTimestamp( const QDateTime ts )
{
    if ( ts.isValid() ) {
        return KGlobal::locale()->formatDateTime( ts );
    }
    else {
        return i18n( "Unknown sending time" );
    }
}


const QString
formatText( const QString& text, const QString& lineSep )
{
    return QStringList::split('\r', text, true).join(lineSep);
}


const QString
substituteTags( const Message* msg,
                const QString& templ,
                const QString& lineSep )
{
    QString t( templ ); // we need a copy of the original template
    return t.replace( SENDER_TAG, formatAddress(msg->sender()) )
            .replace( RECEIVER_TAG, formatAddress(msg->receiver()) )
            .replace( DATE_TAG, formatTimestamp(msg->timestamp()) )
            .replace( TEXT_TAG, formatText(msg->text(), lineSep) );
}


}
