/***************************************************************************
                          messagelistbox.cpp  -  description
                             -------------------
    begin                : Wed Nov 6 2002
    copyright            : (C) 2002 by Michael Schuerig
    email                : michael@schuerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kapplication.h>
#include <kdebug.h>
#include <kglobal.h>
#include <klocale.h>
#include <qregexp.h>
#include <qstring.h>
#include <qpainter.h>
#include <qsimplerichtext.h>

#include "message.h"
#include "messageformatter.h"
#include "msgsender.h"
#include "messagelistbox.h"


namespace {

enum { ID_COPY, ID_DELETE };

class MessageItem : public QListBoxItem {
public:
    MessageItem( Message* msg, const QString& text );
    ~MessageItem();

    Message* message()
        { return message_; }
    
    void paint( QPainter* painter );
    int height( const QListBox* lb ) const;
    int width( const QListBox* lb ) const;
    
private:
    Message* message_;
    mutable QSimpleRichText widget_;
};

}


MessageListBox::MessageListBox( const QString& msgTemplate,
                                QWidget* parent,
                                const char* name )
    : KListBox(parent, name),
      formatter_( new MessageFormatter(msgTemplate) )
{
    clearWFlags( Qt::WStaticContents | Qt::WRepaintNoErase );

    connect( this, SIGNAL(selectionChanged(QListBoxItem*)),
             this, SLOT(emitMessageSelected(QListBoxItem*)) );
}


MessageListBox::~MessageListBox()
{
    delete formatter_;
}


void
MessageListBox::setMessages( const MessageList& messages )
{
    clear();

    for ( Message* msg = const_cast<MessageList&>(messages).first();
          msg != 0;
          msg = const_cast<MessageList&>(messages).next() ) {
        insertItem( new MessageItem(msg, formatter_->format(msg)) );
    }
}

void
MessageListBox::emitMessageSelected( QListBoxItem* msgItem )
{
    emit messageSelected( dynamic_cast<MessageItem*>(msgItem)->message() );
}


const Message*
MessageListBox::selectedMessage() const
{
    int idx = currentItem();
    if ( idx == -1 )
        return 0;

    MessageItem* msgItem
        = dynamic_cast<MessageItem*>( item(idx) );

    return msgItem->message();
}



namespace {

MessageItem::MessageItem( Message* msg, const QString& text )
    : message_(msg),
      widget_(text, KApplication::font())
{
}


MessageItem::~MessageItem()
{
}


void
MessageItem::paint( QPainter *painter )
{
    widget_.setWidth( painter, listBox()->contentsWidth() );
//    widget_.adjustSize();
    QRect view( 0, 0, listBox()->contentsWidth(), widget_.height() );
    widget_.draw( painter, 0, 0, view, listBox()->colorGroup() );
}


int
MessageItem::height( const QListBox* ) const
{
    widget_.setWidth( listBox()->contentsWidth() );
//    widget_.adjustSize();
    return widget_.height();
}


int
MessageItem::width( const QListBox* lb ) const
{
    return lb->contentsWidth();
}

}
