/***************************************************************************
                          messagestore.h  -  description
                             -------------------
    begin                : Tue Oct 29 2002
    copyright            : (C) 2002 by Michael Schuerig
    email                : michael@schuerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MESSAGESTORE_H
#define MESSAGESTORE_H

#include <ksharedptr.h>
#include <qobject.h>

#include "message.h"
#include "msgsender.h"

class QDateTime;
class QString;


/**
 * @author Michael Schuerig
 */
class MessageStore : public QObject, public KShared {
Q_OBJECT
public:
    /**
     * Takes ownership of senderFactory.
     */
    MessageStore( KSharedPtr<MsgSender::Factory> senderFactory,
                  const QString& messagesFile = QString::null );
    ~MessageStore();

    /**
     * Clears the messagestore, reads messages, sets modified to false.
     */
    void readMessages( const QString& messagesFile = QString::null );
    /**
     * Imports messages from a phone.
     * Sets modified to true.
     */
    void importMessagesFromPhone();
    /**
     * Saves the messages and sets modified to false.
     */
    void saveMessages( const QString& messagesFile = QString::null,
                       bool onlyIfModified = true ) const;

    const MessageList messages() const;
    uint messageCount() const;
    const MsgSenderList senders() const;

signals:
    /**
     * Emitted after a wholesale change: readMessages, importMessagesFromPhone.
     */
    void storeChanged();
    void messagesChangedForSender( const MsgSender* sender );
    void senderWasAdded( const MsgSender* sender );
    void progress( unsigned long current, unsigned long max );

public slots:
    /**
     * Sets modified to true.
     */
    void addMessage( MessageConstants::Type type,
                     const QString& phoneNumber, const QDateTime& timestamp,
                     const QString& text, bool onlyIfNew = true );
    /**
     * Sets modified to true.
     */
    void addSentMessage( const QString& receiverNumber, const QString& text );
    /**
     * Sets modified to true.
     */
    void deleteMessage( Message* msg );

private:
    class ReaderProgress;
    friend class ReaderProgress;

    void initMyself();

    MsgSender* findSender( const QString& phoneNumber ) const;
    MsgSender* newSender( const QString& phoneNumber );

    void switchMessagesFile( const QString& messagesFile ) const;
    void backupMessagesFile() const;
    void emitProgress( unsigned long current, unsigned long max );

    mutable bool modified_;
    KSharedPtr<MsgSender::Factory> senderFactory_;
    mutable QString messagesFile_;
    MsgSenderContainer senders_;
    MsgSender* myself_;
    MessageContainer messages_;
    bool reading_;
};

#endif
