/***************************************************************************
                          mobilephonebook.h  -  description
                             -------------------
    begin                : Sat Oct 26 2002
    copyright            : (C) 2002 by Michael Schuerig
    email                : michael@schuerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MOBILEPHONEBOOK_H
#define MOBILEPHONEBOOK_H

#include <ksharedptr.h>
#include <qobject.h>
#include <qstringlist.h>
#include <qvaluelist.h>


class KConfig;


/**
 * @author Michael Schuerig
 */

class MobilePhoneBook : public QObject, public KShared {
Q_OBJECT
public:

    struct Entry {
        Entry() {}
        Entry( const QString& phoneNumber_, const QString name_, const QString uid_ )
            : phoneNumber(phoneNumber_), name(name_), uid(uid_) {}
        QString phoneNumber;
        QString name;
        QString uid;
        bool isValid() { return !phoneNumber.isEmpty(); }
        static const Entry null;
    };
    typedef QValueList<Entry> EntryList;

    MobilePhoneBook( KConfig* config = 0 );
    ~MobilePhoneBook();

    const EntryList entries() const;
    const QStringList recentNumbers() const;
    /**
     * @param number Normalized phone number.
     */
    static const Entry findEntryForNumber( const QString& number );
    static const Entry findMyself();

    void readConfig( KConfig* config );
    void saveConfig( KConfig* config ) const;
    
signals:
    void phoneNumbersChanged();

public slots:
    void rememberAsRecentNumber( const QString& number );
    
private:
    MobilePhoneBook( const MobilePhoneBook& ); // not implemented
    MobilePhoneBook operator=( const MobilePhoneBook& ); // not implemented

    unsigned int recentNumbersCount_;
    QStringList recentNumbers_;
    
};


#endif
