/***************************************************************************
                          msgsender.cpp  -  description
                             -------------------
    begin                : Mon Oct 28 2002
    copyright            : (C) 2002 by Michael Schuerig
    email                : michael@schuerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kdebug.h>

#include "msgsender.h"

namespace {

enum { SENDER_HASH_SIZE = 197 };

}


MsgSender::Factory::~Factory()
{
}


MsgSender::MsgSender( const QString& phoneNumber,
                      const QString& name,
                      const QString& uid )
    : phoneNumber_(phoneNumber), name_(name), uid_(uid)
{
}


MsgSender::~MsgSender()
{
}


void
MsgSender::addReceivedMessage( const Message* msg )
{
    Q_ASSERT(msg != 0);
    receivedMessages_.append(msg);
}

void
MsgSender::removeReceivedMessage( Message* msg )
{
    Q_ASSERT(msg != 0);
    kdDebug() << "MsgSender::removeReceivedMessage: " << endl << msg->toString() << endl;
    bool removed = receivedMessages_.removeRef(msg);
    Q_ASSERT( removed );
}


void
MsgSender::addSentMessage( const Message* msg )
{
    Q_ASSERT(msg != 0);
    sentMessages_.append(msg);
}


void
MsgSender::removeSentMessage( Message* msg )
{
    Q_ASSERT(msg != 0);
    kdDebug() << "MsgSender::removeSentMessage: " << endl << msg->toString() << endl;
    bool removed = sentMessages_.removeRef(msg);
    Q_ASSERT( removed );
}


const QString
MsgSender::toString() const
{
    return phoneNumber_ + ", " + name_ + ", " + uid_;
}


MsgSenderContainer::MsgSenderContainer()
    : numberToSender_( SENDER_HASH_SIZE )
{
    setAutoDelete( true );
}


MsgSenderContainer::~MsgSenderContainer()
{
}


MsgSender*
MsgSenderContainer::newSender( const QString& phoneNumber,
                               const QString& name,
                               const QString& uid )
{
    MsgSender* sender = new MsgSender( phoneNumber, name, uid );
    append( sender );
    numberToSender_.insert( phoneNumber, sender );
    
    return sender;
}


void
MsgSenderContainer::removeSender( MsgSender* sender )
{
    Q_ASSERT( sender != 0 );

    MsgSender* sender2 = numberToSender_.take( sender->phoneNumber() );
    Q_ASSERT( sender == sender2 );

    bool removed = removeRef( sender );
    Q_ASSERT( removed );
}


MsgSender*
MsgSenderContainer::findByPhoneNumber( const QString& phoneNumber ) const
{
    return numberToSender_.find( phoneNumber );
}
