/***************************************************************************
                          msgsender.h  -  description
                             -------------------
    begin                : Mon Oct 28 2002
    copyright            : (C) 2002 by Michael Schuerig
    email                : michael@schuerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MSGSENDER_H
#define MSGSENDER_H

#include <ksharedptr.h>
#include <qdict.h>
#include <qstring.h>
#include <qptrlist.h>

#include "message.h"

class MsgSender;
class MsgSenderList;


/**
 * @author Michael Schuerig
 */
class MsgSender {

    friend class MsgSenderList;
    friend class QPtrList<MsgSender>;
    friend class MsgSenderContainer;
    friend class QDict<MsgSender>;

public:

    struct Factory : public KShared {
        virtual MsgSender* newSender( MsgSenderContainer& container,
                                      const QString& phoneNumber ) = 0;
        virtual MsgSender* newMyselfSender( MsgSenderContainer& container ) = 0;
        virtual ~Factory() = 0;
    };

    const QString phoneNumber() const
        { return phoneNumber_; }
    const QString name() const
        { return name_; }
    const QString uid() const
        { return uid_; }

    uint receivedMessageCount() const
        { return receivedMessages_.count(); }
    const MessageList& receivedMessages() const
        { return receivedMessages_; }
    MessageList& receivedMessages()
        { return receivedMessages_; }
    void addReceivedMessage( const Message* msg );
    void removeReceivedMessage( Message* msg );

    uint sentMessageCount() const
        { return sentMessages_.count(); }
    const MessageList& sentMessages() const
        { return sentMessages_; }
    MessageList& sentMessages()
        { return sentMessages_; }
    void addSentMessage( const Message* msg );
    void removeSentMessage( Message* msg );
    
    const QString toString() const;
    
private:
    MsgSender() {}
    MsgSender( const QString& phoneNumber,
               const QString& name = QString::null,
               const QString& uid = QString::null );
    ~MsgSender();

    MsgSender( const MsgSender& ); // not implemented
    MsgSender& operator=( const MsgSender& ); // not implemented

    const QString phoneNumber_;
    const QString name_;
    const QString uid_;
    MessageList receivedMessages_;
    MessageList sentMessages_;
};


struct MsgSenderList : public QPtrList<MsgSender> {
};


class MsgSenderContainer : public MsgSenderList {
public:
    MsgSenderContainer();
    ~MsgSenderContainer();
    
    MsgSender* newSender( const QString& phoneNumber,
                          const QString& name = QString::null,
                          const QString& uid = QString::null );
    void removeSender( MsgSender* sender );

    /**
     * @param phoneNumber Normalized phone number.
     */
    MsgSender* findByPhoneNumber( const QString& phoneNumber ) const;

    MsgSender* find( const MsgSender ) const;
    
private:
    QDict<MsgSender> numberToSender_;
};


#endif
