/***************************************************************************
                          phoneaccess.cpp  -  description
                             -------------------
    begin                : Wed Nov 6 2002
    copyright            : (C) 2002 by Michael Schuerig
    email                : michael@schuerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kdebug.h>
#include <qdatetime.h>
 
#include <gsmlib/gsm_unix_serial.h>
#include <gsmlib/gsm_me_ta.h>
#include <gsmlib/gsm_sorted_sms_store.h>
#include <gsmlib/gsm_util.h>

#include "phoneaccess.h"

using namespace gsmlib;

namespace {

Ref<MeTa> makePhoneMeTa( const PhoneAccess::Config& config );
QDateTime gsmTimestampToDateTime( const gsmlib::Timestamp& ts );

/*
class Progress : public ProgressBase {
public:
    virtual void reportProgress(int part, int total)
    {
        kdDebug() << "Progress " << QString::number(part) << " of " << QString::number(total) << endl;
    }
};
*/

}



PhoneAccess::PhoneAccess()
{
}


PhoneAccess::~PhoneAccess()
{
}


PhoneAccess*
PhoneAccess::instance()
{
    static PhoneAccess* instance = new PhoneAccess();
    return instance;
}


void
PhoneAccess::setConfig( const Config& config )
{
    kdDebug() << "PhoneAccess::setConfig " << config.device << endl;

    Q_ASSERT( !config.device.isEmpty() );
    Q_ASSERT( !config.baudrate.isEmpty() );
    Q_ASSERT( !config.smsStore.isEmpty() );

    config_ = config;
}


void
PhoneAccess::retrieveMessages()
{
    kdDebug() << "PhoneAccess::retrieveMessages" << endl;

    try {
        Ref<MeTa> phone( makePhoneMeTa(config_) );
        SortedSMSStoreRef sourceStore(
            new SortedSMSStore(phone->getSMSStore(config_.smsStore.latin1())) );

        for ( SortedSMSStore::iterator it = sourceStore->begin();
              it != sourceStore->end(); ++it ) {
            SMSMessageRef msg = it->message();

            QString number( QString::fromLatin1(msg->address().toString().c_str()) );
            QDateTime timestamp( gsmTimestampToDateTime(msg->serviceCentreTimestamp()) );
            QString text( QString::fromLatin1(msg->userData().c_str()) );

            switch ( msg->messageType() ) {
            case SMSMessage::SMS_DELIVER:
                kdDebug() << "got deliver message" << endl;
                emit retrievedMessage( MessageConstants::SMS_DELIVER,
                                       number, timestamp, text );
                break;
            case SMSMessage::SMS_SUBMIT:
                kdDebug() << "got submit message" << endl;
                emit retrievedMessage( MessageConstants::SMS_SUBMIT,
                                       number, timestamp, text );
                break;
            default:
                kdWarning() << "SMS message type unrecognized: " << msg->messageType() << endl;
                break;
            }
        }
    }
    catch ( gsmlib::GsmException& ex ) {
        kdError() << "PhoneAccess::iterateStoredMessages: " << ex.what() << endl;
    }
    catch (...) {
        kdError() << "PhoneAccess::iterateStoredMessages: "
                  << "An unknown exception has occurred while reading from the phone."
                  << endl;
    }
}


void
PhoneAccess::sendMessage( const QString& number,
                          const QString& text )
{

    try {
        Ref<MeTa> phone( makePhoneMeTa(config_) );
        Ref<SMSSubmitMessage> submitSMS(new SMSSubmitMessage(text.latin1(),
                                                             number.latin1()) );
        phone->sendSMS( submitSMS );
    }
    catch ( gsmlib::GsmException& ex ) {
        kdError() << "PhoneAccess::sendMessage: " << ex.what() << endl;
    }
    catch (...) {
        kdError() << "PhoneAccess::sendMessage: "
                  << "An unknown exception has occurred while sending a message."
                  << endl;
    }
}



namespace {

Ref<MeTa>
makePhoneMeTa( const PhoneAccess::Config& config )
{
    kdDebug() << "makePhoneMeTa " << endl;

    Q_ASSERT( !config.device.isEmpty() );
    Q_ASSERT( !config.baudrate.isEmpty() );
    Q_ASSERT( !config.smsStore.isEmpty() );
    
    std::string device( config.device.latin1() );
    speed_t     baudrate( baudRateStrToSpeed(config.baudrate.latin1()) );
    std::string initString( !config.initString.isEmpty()
                                ? config.initString.latin1()
                                : DEFAULT_INIT_STRING );
    std::string smsStore( config.smsStore.latin1() );
    bool swHandshake = false;

    Ref<Port> port(
        new UnixSerialPort(device, baudrate, initString, swHandshake) );
    port->setTimeOut( config.timeout );
    return Ref<MeTa>( new MeTa(port) );
}


QDateTime
gsmTimestampToDateTime( const gsmlib::Timestamp& ts )
{
    static const QDateTime NULL_TIMESTAMP;

    if ( ts.empty() ) {
//        kdWarning() << "EMPTY TIMESTAMP" << endl;
        return NULL_TIMESTAMP;
    }
    else {
        QDate date( (ts._year >= 2000) ? ts._year : ts._year + 2000,
                     ts._month, ts._day );
        //### take time zone into account
        QTime time( ts._hour, ts._minute, ts._seconds );
        return QDateTime( date, time );
    }
}

}
