/***************************************************************************
                          senderlistbox.cpp  -  description
                             -------------------
    begin                : Wed Nov 6 2002
    copyright            : (C) 2002 by Michael Schuerig
    email                : michael@schuerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kdebug.h>
#include <klistview.h>
#include <klocale.h>
#include <qstring.h>
 
#include "message.h"
#include "msgsender.h"
 
#include "senderlistbox.h"


namespace {

QString displayTextFor( const MsgSender* sender )
{
    if ( !sender->name().isEmpty() )
        return sender->name();
    else
        return sender->phoneNumber();
}

class SenderItem : public KListViewItem {
public:
    SenderItem( QListView* parent, const MsgSender* sender )
        : KListViewItem(parent,
                        displayTextFor(sender), sender->phoneNumber()),
          sender_(sender) {}
    const MsgSender* sender() const
        { return sender_; }
private:
    const MsgSender* sender_;
};

}


SenderListBox::SenderListBox( bool phoneNumberVisible,
                              QWidget* parent,
                              const char* name )
    : KListView( parent, name )
{
    setHScrollBarMode( AlwaysOff );
    setAllColumnsShowFocus( true );
    addColumn( i18n("Name") );
    setPhoneNumberVisible( phoneNumberVisible );
    
    connect( this, SIGNAL(selectionChanged(QListViewItem*)),
             this, SLOT(emitSenderSelected(QListViewItem*)) );
}


SenderListBox::~SenderListBox()
{
}


bool
SenderListBox::isPhoneNumberVisible()
{
    return columns() > 1;
}


void
SenderListBox::setPhoneNumberVisible( bool visible )
{
    if ( visible == isPhoneNumberVisible() )
        return;

    if ( visible ) {
        addColumn( i18n("PhoneNumber") );
    }
    else {
        removeColumn( 1 );
    }
}


void
SenderListBox::setSenders( const MsgSenderList& senders )
{
    clear();

    for ( const MsgSender* sender = const_cast<MsgSenderList&>(senders).first();
          sender != 0;
          sender = const_cast<MsgSenderList&>(senders).next() ) {
        new SenderItem( this, sender );
    }
}


void
SenderListBox::selectSender( const QString& uid )
{
    for ( QListViewItem* item = firstChild();
          item != 0;
          item = item->nextSibling() ) {
        const MsgSender* sender = dynamic_cast<const SenderItem*>(item)->sender();
        if ( sender->uid() == uid ) {
            setSelected( item, true );
            return;
        }
    }
}


const MsgSender*
SenderListBox::selectedSender() const
{
    return dynamic_cast<SenderItem*>(currentItem())->sender();
}


void
SenderListBox::emitSenderSelected( QListViewItem* senderItem )
{
    emit senderSelected( dynamic_cast<SenderItem*>(senderItem)->sender() );
}
