/***************************************************************************
                          threadedphoneaccess.cpp  -  description
                             -------------------
    begin                : Sat Nov 23 2002
    copyright            : (C) 2002 by Michael Schuerig
    email                : michael@schuerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kdebug.h>
#include <qdatetime.h>

#include "phoneaccess.h"

#include "threadedphoneaccess.h"


class ThreadedPhoneAccess::SlotRequest_retrieveMessages
    : public KActiveObjectProxy::SlotRequest {
public:
    SlotRequest_retrieveMessages( PhoneAccess* phone )
        : obj_( phone ) {}
    void operator()()
        { obj_->retrieveMessages(); }
private:
    PhoneAccess* obj_;
};


class ThreadedPhoneAccess::SlotRequest_sendMessage
    : public KActiveObjectProxy::SlotRequest {
public:
    SlotRequest_sendMessage( PhoneAccess* phone,
                             const QString& number,
                             const QString& text )
        : obj_(phone), param1_(number), param2_(text) {}
    void operator()()
        { obj_->sendMessage( param1_, param2_ ); }
private:
    PhoneAccess* obj_;
    const QString param1_;
    const QString param2_;
};


class ThreadedPhoneAccess::SlotRequest_setConfig
    : public KActiveObjectProxy::SlotRequest {
public:
    SlotRequest_setConfig( PhoneAccess* phone,
                           const PhoneAccess::Config& config )
        : obj_(phone), param1_(config) {}
    void operator()()
        { obj_->setConfig( param1_ ); }
private:
    PhoneAccess* obj_;
    const PhoneAccess::Config param1_;
};


class ThreadedPhoneAccess::SignalRequest_retrievedMessage
    : public KActiveObjectProxy::SignalRequest {
public:
    SignalRequest_retrievedMessage( MessageConstants::Type type,
                                    const QString& number,
                                    const QDateTime& timestamp,
                                    const QString& text )
        : param1_(type), param2_(number), param3_(timestamp), param4_(text) {}
    void doEmit( KActiveObjectProxy* proxy )
        { dynamic_cast<ThreadedPhoneAccess*>(proxy)->emitSignal_retrievedMessage(
            param1_, param2_, param3_, param4_ ); }
private:
    const MessageConstants::Type param1_;
    const QString param2_;
    const QDateTime param3_;
    const QString param4_;
};



ThreadedPhoneAccess::ThreadedPhoneAccess(PhoneAccess* phone,
                                         QObject* parent,
                                         const char* name)
    : KActiveObjectProxy( phone, parent, name )
{
    connect( object(), SIGNAL(retrievedMessage(MessageConstants::Type,
                                               const QString&,
                                               const QDateTime&, const QString&)),
             this, SLOT(enqueueSignal_retrievedMessage(MessageConstants::Type,
                                                       const QString&,
                                                       const QDateTime&,
                                                       const QString&)) );
}


ThreadedPhoneAccess*
ThreadedPhoneAccess::instance()
{
    static ThreadedPhoneAccess* instance
        = new ThreadedPhoneAccess( PhoneAccess::instance() );
    return instance;
}


ThreadedPhoneAccess::~ThreadedPhoneAccess()
{
}


void
ThreadedPhoneAccess::retrieveMessages()
{
    kdDebug() << "ThreadedPhoneAccess::retrieveMessages" << endl;
    enqueueSlot( new SlotRequest_retrieveMessages(dynamic_cast<PhoneAccess*>(object())) );
}


void
ThreadedPhoneAccess::sendMessage( const QString& number,
                                  const QString& text )
{
    kdDebug() << "ThreadedPhoneAccess::sendMessage" << endl;
    enqueueSlot( new SlotRequest_sendMessage(dynamic_cast<PhoneAccess*>(object()),
                                             number, text) );
}


void
ThreadedPhoneAccess::setConfig( const PhoneAccess::Config& config )
{
    kdDebug() << "ThreadedPhoneAccess::setConfig" << endl;
    enqueueSlot( new SlotRequest_setConfig(dynamic_cast<PhoneAccess*>(object()),
                                           config) );
}


void
ThreadedPhoneAccess::emitSignal_retrievedMessage( MessageConstants::Type type,
                                                  const QString& number,
                                                  const QDateTime& timestamp,
                                                  const QString& text )
{
    kdDebug() << "ThreadedPhoneAccess::emitSignal_retrievedMessage" << endl;
    emit retrievedMessage( type, number, timestamp, text );
}


void
ThreadedPhoneAccess::enqueueSignal_retrievedMessage( MessageConstants::Type type,
                                                     const QString& number,
                                                     const QDateTime& timestamp,
                                                     const QString& text )
{
    kdDebug() << "ThreadedPhoneAccess::enqueueSignal_retrievedMessage" << endl;
    enqueueSignal( new SignalRequest_retrievedMessage(type, number,
                                                      timestamp, text) );
}
