/***************************************************************************
                          xmlinputsourcewithprogress.cpp  -  description
                             -------------------
    begin                : Sat Nov 9 2002
    copyright            : (C) 2002 by Michael Schuerig
    email                : michael@schuerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qglobal.h>
#include <qiodevice.h>

#include "progresscallback.h"
   
#include "xmlinputsourcewithprogress.h"



XmlInputSourceWithProgress::XmlInputSourceWithProgress( QIODevice* dev,
                                                        ProgressCallback* progress )
    : QXmlInputSource( dev ),
      pos_( 0 ), max_( dev->size() ),
      progress_( progress )
{
    Q_ASSERT( progress_ != 0 );

    interval_ = progress_->interval();
    if ( interval_ == 0 )
        interval_ = max_ / 100;
    if ( interval_ == 0 )
        interval_ = 1;
}


XmlInputSourceWithProgress::~XmlInputSourceWithProgress()
{
}


QChar
XmlInputSourceWithProgress::next()
{
    ++pos_;

    if ( pos_ % interval_ == 0 ) {
        progress_->progress( pos_, max_ );
    }
    
    return QXmlInputSource::next();
}


void
XmlInputSourceWithProgress::reset()
{
    pos_ = 0;
}

